import { IPermission, IRole, InferPermissions, InferResources, RoleAddResult, RoleRemoveResult, RoleUpdateResult, TRoleOptions, ToObjectResult, addOptions, removeOptions, updateOptions } from '../types';
/**
 * `Role` class
 */
declare class Role<T extends TRoleOptions> implements IRole<T> {
    name: T['name'];
    description?: string;
    meta?: Record<string, any>;
    config: T['config'];
    resources: InferResources<T['config']>[];
    permissions: InferPermissions<T['config']>;
    constructor(options: T);
    private getDefaultPermission;
    private init;
    getPermissions(): InferPermissions<T["config"]>;
    getResources(): InferResources<T['config']>[];
    add<S extends string, P extends IPermission>(options: addOptions<S, P>): RoleAddResult<T, S, P>;
    remove<S extends keyof this['permissions'] & (string & {})>(options: removeOptions<S>): RoleRemoveResult<T, S>;
    update<S extends keyof this['permissions'] & (string & {}), P extends IPermission>(options: updateOptions<S, P>): RoleUpdateResult<T, S, P>;
    can<S extends keyof this['permissions'], P extends keyof this['permissions'][S]>(resource: S, permission: P): boolean;
    cannot<S extends keyof this['permissions'], P extends keyof this['permissions'][S]>(resource: S, permission: P): boolean;
    canAny<S extends keyof this['permissions']>(resource: S, permissions: (keyof this['permissions'][S])[]): boolean;
    canAll<S extends keyof this['permissions']>(resource: S, permissions: (keyof this['permissions'][S])[]): boolean;
    clone(): Role<T>;
    toJSON(): string;
    toObject(): ToObjectResult<T>;
    /**
     *  Creates a role from a JSON string
     */
    static fromJSON<T extends TRoleOptions>(json: string): Role<{
        config: T["config"];
        description: T["description"];
        meta: T["meta"];
        name: T["name"];
    }>;
    /**
     *  Creates a role from object
     */
    static fromObject<T extends TRoleOptions>(object: T): Role<{
        config: T["config"];
        description: T["description"];
        meta: T["meta"];
        name: T["name"];
    }>;
    /**
     * Validates the role object
     */
    static validate(object: any, cb?: (result: boolean, errors: string[]) => void): string[];
    static validate(object: any): string[];
}
export default Role;
